# -*- coding: utf-8 -*-

# For logging use 
# python3 tutorial.py --nodblog --loglevel debug --verbose all --logfile /tmp/my_logfile


from MythTV import MythTVService
from MythTV import datetime
from datetime import timedelta

import logging                         # optional the services-api transfers
import os
from pprint import pprint
from lxml   import etree


if __name__ == "__main__":

    host=("127.0.0.1")             # backend
    hostname = os.uname()[1]       # backend's hostname
    
    ## enable logging for debug messages of the services-api transfers: 
    #logging.basicConfig(level=logging.DEBUG)
    #logging.getLogger('requests.packages.urllib3').setLevel(logging.WARNING)
    #logging.getLogger('urllib3.connectionpool').setLevel(logging.WARNING)    
    
    # Use cases:
    
    if 1:
        """ 
        Use case: toggle 'DVBEITScan' value for the first capture card.
        Note: 'DVBEITScan' is a boolean
        """
        # Initialize the service:
        service = 'Capture'
        s = MythTVService(service, host)

        print("Available operations: %s" %(s.list_operations()))
        ## Available operations: ['AddCaptureCard', 'AddCardInput', 'GetCaptureCard',....]
        ## 'GetCaptureCardList', 'RemoveCaptureCard', 'RemoveCardInput', 'UpdateCaptureCard',
        ## 'UpdateCardInput']
        
        # The long way: make every step in a chain sequentially:
        operation = 'GetCaptureCardList'
        defop = s.get_operation(operation)
        print(defop)
        ## {'opname': 'GetCaptureCardList', 'optype': 'GET', 'opdata': {'HostName': '', 'CardType': ''}}
        
        defop['opdata']['HostName'] = hostname
        defop_stripped = s.strip_operation_defaults(operation, defop)
        print(defop_stripped)
        ## {'opname': 'GetCaptureCardList', 'optype': 'GET', 'opdata': {'HostName': 'localhost'}}
        
        # Note: no need to translate opdata dictionary from python types to strings
        
        if s.perform_operation(defop_stripped):
            # 's' is now like an object of type 'CaptureCardList':        
            DVBEITScan_s1 = s.capturecards[0].dvbeitscan
            print("DVBEITScan_s1 : %s" %DVBEITScan_s1)
            ## DVBEITScan_s1 : True
        
        
        # now update the 'DVBEITScan' value:
        operation = 'UpdateCaptureCard'
        defop = s.get_operation(operation)
        print(defop)
        ## {'opname': 'UpdateCaptureCard', 'optype': 'POST', 
        ##  'opdata': {'CardId': 0, 'Setting': '', 'Value': ''}}
        
        # Fill in the correct values:
        defop['opdata']['CardId'] = 1
        defop['opdata']['Setting'] = 'dvb_eitscan'
        defop['opdata']['Value'] = not DVBEITScan_s1

        # Translate 'POST' operations before sending:
        encop = s.encode_operation(defop)
        print(defop)
        ## {'opname': 'UpdateCaptureCard', 'optype': 'POST',
        ##  'opdata': {'CardId': '1', 'Setting': 'dvb_eitscan', 'Value': '0'}}
            
        res = s.perform_operation(encop)
        print("POST operation returned '%s'" %res)
        ## POST operation returned 'True'
        
        
        # Check the setting by using the fast way:
        opdata = { 'HostName' : hostname }

        sq = MythTVService(service, host, opname='GetCaptureCardList', optype="GET", **opdata)
        # 'sq' is now like an object of type 'CaptureCardList':
        #pprint(sq.capturecards)
        assert DVBEITScan_s1 is not sq.capturecards[0].dvbeitscan, "Quick way and long way results are not equal!"
        
        
    if 1:
        """ 
        Use case: get the program list for the next day with a defined pagesize.
        Note: page size is defined by the number of entries
        """
        service = 'Guide'
        operation = 'GetProgramList'

        s = MythTVService(service, host)
        
        # get the default operations's dictionary:
        def_optdict = s.get_operation(operation)
        print(def_optdict)
        ## {'opname': 'GetProgramList', 'optype': 'GET', 
        ## 'opdata': {'StartIndex': 0, 'Count': 0, 'StartTime': datetime(1900, 1, 1, 0, 0, tzinfo=<....>),
        ##            'EndTime': datetime(1900, 1, 1, 0, 0, tzinfo=<.....>), 'ChanId': 0, 'TitleFilter': '',
        ##            'CategoryFilter': '', 'PersonFilter': '', 'KeywordFilter': '', 'OnlyNew': '0', 
        ##            'Details': '0', 'Sort': '', 'Descending': '0', 'WithInvisible': '0'}}

        starttime = datetime.now()
        endtime   = starttime + timedelta(days=1, hours=1)
        # convert to mythtv's 'datetime' class:
        endtime   = datetime.duck(endtime)
        
        # define the operation's dict based on the default operation's dictionary
        opdict = {}
        opdict['opname'] = operation
        opdict['optype'] = 'GET'
        opdict['opdata'] =  {}
        opdict['opdata']['StartIndex'] = 0
        opdict['opdata']['Count'] = 200              # this defines the page size
        opdict['opdata']['StartTime'] = starttime
        opdict['opdata']['EndTime'] = endtime
        opdict['opdata']['Details'] = True

        # get_pages returns an iterator
        progs = s.get_pages(opdict, 'Programs')
        while True:
            try:
                ps = next(progs)
                for p in ps:
                    print(p.title)
            except StopIteration:
                break      
                
        # One will get something like this:
        ## How I Met Your Mother
        ## Young Sheldon        
        ## .....

        
    if 1:  
        """
        Use Case: Get a 'program' like object of a recorded entry.
        Evaluate the recstatus of the recorded entry.
        """
        
        service = 'Dvr'                 # uses enumeration RecStatus.Type
        operation = 'GetRecorded'       # GET  # returns a 'Program'

        s = MythTVService(service, host)

        opdict = {}
        opdict['opname'] = operation
        opdict['optype'] = 'GET'
        opdict['opdata'] =  {}
        opdict['opdata']['RecordedId'] = 16

        result = s.perform_operation(opdict)
        print(result)          # True
        
        # Get the version of this operation:
        print("Operation Version : %s" %s.operation_version)             #  1.11

        print(s._field_order)      # this prints all availabble properties ['StartTime', 'EndTime', 'Title', ....}
        print(s.title)             # 'ZaB Night'
        print(s.starttime)         # 2020-02-21 22:54:13+01:00    # local time plus offset to UTC
        print(s.channel.chanid)    # 10101 
        print(s.videoprops)        # 46  0x2E   # defined as "xs:int" in the xsd
        print(s.audioprops)        # 0          # defined as "xs:int" in the xsd
        print(s.programflags)      # 72         # defined as "xs:int" in the xsd
        print(s.recording)         # class of type "RecordingInfo"
        print(s.recording.status)  # 'Recorded'
        
        # get the definition of the 'RecStatus':
        print(s.schema_dict["RecStatus.Type"])       # {'Pending': -15, 'Failing': -14, 'MissedFuture': -11,....}
        print(s.schema_dict["RecStatus.Type"][s.recording.status])        # -3
               
               
    if 0:
        """
        Use Case: Display the wsdl file, the schema (xsd) and the corresponding 
        schema dictionary of a service.
        Note: This creates a huge amount of data.
        """
        
        service = 'Dvr'
        s = MythTVService(service, host)
        
        print(etree.tostring(s.wsdl, pretty_print=True, encoding='unicode'))
        print(s.list_schema())
        pprint(s.schema_dict)
        
